@@ -5,6 +5,7 @@ from ._embedding_base import EmbeddingModelBase
 from ._embedding_usage import EmbeddingUsage
 from ._embedding_response import EmbeddingResponse
 from ._dashscope_embedding import DashScopeTextEmbedding
+from ._dashscope_multimodal_embedding import DashScopeMultiModalEmbedding
 from ._openai_embedding import OpenAITextEmbedding
 from ._gemini_embedding import GeminiTextEmbedding
 from ._ollama_embedding import OllamaTextEmbedding
@@ -17,6 +18,7 @@ __all__ = [
     "EmbeddingUsage",
     "EmbeddingResponse",
     "DashScopeTextEmbedding",
+    "DashScopeMultiModalEmbedding",
     "OpenAITextEmbedding",
     "GeminiTextEmbedding",
     "OllamaTextEmbedding",
