@@ -1,14 +1,9 @@
 # -*- coding: utf-8 -*-
 """The embedding model base class."""
-from typing import Any, List, TYPE_CHECKING
+from typing import Any
 
 from ._embedding_response import EmbeddingResponse
 
-if TYPE_CHECKING:
-    from ..model import ChatResponse
-else:
-    ChatResponse = "ChatResponse"
-
 
 class EmbeddingModelBase:
     """Base class for embedding models."""
@@ -16,21 +11,31 @@ class EmbeddingModelBase:
     model_name: str
     """The embedding model name"""
 
+    supported_modalities: list[str]
+    """The supported data modalities, e.g. "text", "image", "video"."""
+
+    dimensions: int
+    """The dimensions of the embedding vector."""
+
     def __init__(
         self,
         model_name: str,
+        dimensions: int,
     ) -> None:
         """Initialize the embedding model base class.
 
         Args:
             model_name (`str`):
                 The name of the embedding model.
+            dimensions (`int`):
+                The dimension of the embedding vector.
         """
         self.model_name = model_name
+        self.dimensions = dimensions
 
     async def __call__(
         self,
-        text: List[str],
+        *args: Any,
         **kwargs: Any,
     ) -> EmbeddingResponse:
         """Call the embedding API with the given arguments."""
