@@ -1,15 +1,22 @@
 # -*- coding: utf-8 -*-
 """The memory module."""
 
-from ._memory_base import MemoryBase
 from ._in_memory_memory import InMemoryMemory
 from ._long_term_memory_base import LongTermMemoryBase
 from ._mem0_long_term_memory import Mem0LongTermMemory
-
+from ._memory_base import MemoryBase
+from ._reme import (
+    ReMePersonalLongTermMemory,
+    ReMeTaskLongTermMemory,
+    ReMeToolLongTermMemory,
+)
 
 __all__ = [
     "MemoryBase",
     "InMemoryMemory",
     "LongTermMemoryBase",
     "Mem0LongTermMemory",
+    "ReMePersonalLongTermMemory",
+    "ReMeTaskLongTermMemory",
+    "ReMeToolLongTermMemory",
 ]
