@@ -1,29 +1,31 @@
 # -*- coding: utf-8 -*-
-"""The message module of AgentScope."""
+"""The message module in agentscope."""
 
-from .msg import (
-    Msg,
-)
-
-from .block import (
+from ._message_block import (
+    ContentBlock,
+    TextBlock,
+    ThinkingBlock,
     ToolUseBlock,
     ToolResultBlock,
-    TextBlock,
     ImageBlock,
     AudioBlock,
     VideoBlock,
-    FileBlock,
-    ContentBlock,
+    Base64Source,
+    URLSource,
 )
+from ._message_base import Msg
+
 
 __all__ = [
-    "Msg",
-    "ToolUseBlock",
-    "ToolResultBlock",
     "TextBlock",
+    "ThinkingBlock",
+    "Base64Source",
+    "URLSource",
     "ImageBlock",
     "AudioBlock",
     "VideoBlock",
-    "FileBlock",
+    "ToolUseBlock",
+    "ToolResultBlock",
     "ContentBlock",
+    "Msg",
 ]
