@@ -4,12 +4,15 @@
 from dataclasses import dataclass, field
 from typing import Literal, Sequence
 
-from anthropic.types import ThinkingBlock
-
 from ._model_usage import ChatUsage
 from .._utils._common import _get_timestamp
 from .._utils._mixin import DictMixin
-from ..message import TextBlock, ToolUseBlock
+from ..message import (
+    TextBlock,
+    ToolUseBlock,
+    ThinkingBlock,
+    AudioBlock,
+)
 from ..types import JSONSerializableObject
 
 
@@ -17,7 +20,7 @@ from ..types import JSONSerializableObject
 class ChatResponse(DictMixin):
     """The response of chat models."""
 
-    content: Sequence[TextBlock | ToolUseBlock | ThinkingBlock]
+    content: Sequence[TextBlock | ToolUseBlock | ThinkingBlock | AudioBlock]
     """The content of the chat response, which can include text blocks,
     tool use blocks, or thinking blocks."""
 
@@ -33,7 +36,7 @@ class ChatResponse(DictMixin):
     usage: ChatUsage | None = field(default_factory=lambda: None)
     """The usage information of the chat response, if available."""
 
-    metadata: JSONSerializableObject | None = field(
+    metadata: dict[str, JSONSerializableObject] | None = field(
         default_factory=lambda: None,
     )
     """The metadata of the chat response"""
