@@ -4,7 +4,11 @@ complex workflows and multi-agent conversations."""
 
 from ._msghub import MsgHub
 from ._class import SequentialPipeline, FanoutPipeline
-from ._functional import sequential_pipeline, fanout_pipeline
+from ._functional import (
+    sequential_pipeline,
+    fanout_pipeline,
+    stream_printing_messages,
+)
 
 __all__ = [
     "MsgHub",
@@ -12,4 +16,5 @@ __all__ = [
     "sequential_pipeline",
     "FanoutPipeline",
     "fanout_pipeline",
+    "stream_printing_messages",
 ]
